Exception: DSPy::InstructionUpdateError

Inherits:
Error
  • Object
show all
Defined in:
lib/dspy/errors.rb

Class Method Summary collapse

Class Method Details

.missing_examples_capability(module_class) ⇒ Object



41
42
43
44
45
46
47
# File 'lib/dspy/errors.rb', line 41

def self.missing_examples_capability(module_class)
  new(<<~MESSAGE)
    #{module_class} must implement `with_examples(few_shot_examples)` to support DSPy teleprompters.

    Update the module to return a new instance with the provided examples, or opt out of example optimization flows.
  MESSAGE
end

.missing_instruction_capability(module_class) ⇒ Object



33
34
35
36
37
38
39
# File 'lib/dspy/errors.rb', line 33

def self.missing_instruction_capability(module_class)
  new(<<~MESSAGE)
    #{module_class} must implement `with_instruction(new_instruction)` to support DSPy teleprompters.

    Update the module to return a new instance with the provided instruction, or opt out of teleprompter optimizers.
  MESSAGE
end